package com.fs.store.service.handler;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.fs.common.core.domain.R;
import com.fs.common.utils.IPUtils;
import com.fs.common.utils.ServletUtils;
import com.fs.huifuPay.domain.HuiFuCreateOrder;
import com.fs.huifuPay.domain.HuifuCreateOrderResult;
import com.fs.huifuPay.dto.*;
import com.fs.store.constants.StoreConstants;
import com.fs.store.domain.FsStorePayment;
import com.fs.store.domain.TzConfigInfoDTO;
import com.fs.store.service.handler.param.PayProcessContext;
import com.fs.system.service.ISysConfigService;
import com.fs.system.service.impl.SysConfigServiceImpl;
import com.fs.tzBank.TzBankService;
import com.fs.tzBank.impl.TzBankServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.concurrent.TimeUnit;

import static org.springframework.beans.factory.config.ConfigurableBeanFactory.SCOPE_PROTOTYPE;

/**
 * tzbk支付处理
 */
@Service
@Slf4j
@Scope(value = SCOPE_PROTOTYPE)
public class TzbkPaymentHandler extends PaymentHandler{


    @Autowired
    private ISysConfigService configService;
    @Autowired
    private TzBankService tzBankService;


    @Override
    protected R doProcess(PayProcessContext context) {
        String json = configService.selectConfigByKey("store.pay");
        TzConfigInfoDTO tzConfigInfoDTO = JSON.parseObject(json, TzConfigInfoDTO.class);

        TzPayCreateOrderReqBodyDTO reqBodyDTO = new TzPayCreateOrderReqBodyDTO();
        RequestDTO<TzPayCreateOrderReqBodyDTO> requestDTO = new RequestDTO<>();
        reqBodyDTO.setOrderNo(String.valueOf(context.getPayCode()));
        reqBodyDTO.setOrderSource("02");
        reqBodyDTO.setTrxAmt(this.fsStorePayment.getPayMoney());
        reqBodyDTO.setCurrency("CNY");
        reqBodyDTO.setTransType("2001");
        reqBodyDTO.setTradeMerCstNo(tzConfigInfoDTO.getPlatMerCstNo());
        reqBodyDTO.setPlatMerCstNo(tzConfigInfoDTO.getPlatMerCstNo());
        reqBodyDTO.setBusinessCstNo(user.getUserId().toString());
        reqBodyDTO.setPayNotifyUrl(tzConfigInfoDTO.getPayNotifyUrl());
        reqBodyDTO.setPayerName(user.getUsername());
        reqBodyDTO.setPayerMobileNo(user.getPhone());
        reqBodyDTO.setGoodsInfo(context.getGoodsInfo());
        // 不需要分账
        reqBodyDTO.setNeedLedger("00");
        reqBodyDTO.setClientIp(IPUtils.getIpAddr(ServletUtils.getRequest()));

        // 获取当前时间戳
        Instant currentTimestamp = Instant.now();
        // 增加一个小时
        Instant oneHourLater = currentTimestamp.plusSeconds(3600);
        reqBodyDTO.setExpiredTime(String.valueOf(oneHourLater.toEpochMilli()));


        requestDTO.setReqHeader(TzReqHeaderDTO.getInstance(context.getPayCode()));
        requestDTO.setReqBody(reqBodyDTO);
        try {
            // 创建订单
            TzReqResultDTO<TzPayCreateOrderRspDTO> payCreateOrder = tzBankService.payCreateOrder(requestDTO);
            // 获得小程序请求码
            if("00000000".equals(payCreateOrder.getRetCode())){
                TzPayCreateOrderRspDTO body = payCreateOrder.getBody();

                RequestDTO<TzMiniProgramCodeReqDTO> reqDTORequestDTO = new RequestDTO<>();
                TzMiniProgramCodeReqDTO tzMiniProgramCodeReqDTO = new TzMiniProgramCodeReqDTO();
                tzMiniProgramCodeReqDTO.setOrderNo(body.getOrderFlowNo());
                tzMiniProgramCodeReqDTO.setPlatMerCstNo(tzConfigInfoDTO.getPlatMerCstNo());
                tzMiniProgramCodeReqDTO.setBusinessCstNo(user.getUserId().toString());
                reqDTORequestDTO.setReqHeader(TzReqHeaderDTO.getInstance(context.getPayCode()));
                reqDTORequestDTO.setReqBody(tzMiniProgramCodeReqDTO);
                TzReqResultDTO<TzMiniProgramCodeRespDTO> miniProgramCode = tzBankService.miniProgramCode(reqDTORequestDTO);
                if("00000000".equals(miniProgramCode.getRetCode())){
                    FsStorePayment mt=new FsStorePayment();
                    mt.setPaymentId(context.getPaymentId());
                    mt.setTradeNo(miniProgramCode.getBody().getOrderNo());
                    this.fsStorePaymentService.updateFsStorePayment(mt);
                    TzMiniProgramCodeRespDTO result = miniProgramCode.getBody();
                    result.setPayCode(context.getPayCode());
                    return R.ok().put("payType",context.getPayType()).put("result",result);
                } else {
                    return R.error(miniProgramCode.getRetMsg());
                }
            } else {
                return R.error(payCreateOrder.getRetMsg());
            }
        } catch (Throwable e) {
            log.error("系统异常",e);
            return R.error("系统异常，请联系管理员!");
        }
    }

    @Override
    public boolean support(String code) {
        return "tzbk".equals(code);
    }
}
